#include <iostream>
#include "EventHandler.h"
using namespace std;

const Event LINK_1_BROKEN = 1;
const Event LINK_2_BROKEN = 2;
const Event CONGESTION = 3;

//Set up a chain of event handlers

EventHandler::EventHandler (
	EventHandler* h, Event t
	) : _successor(h), _event(t) { }

const bool EventHandler::HasEventSupport(Event incomingEvent) {
	return _event == incomingEvent;
}

void EventHandler::SetHandler(EventHandler* h, Event t) {
	_successor = h;
	_event = t;
}

void EventHandler::HandleEvent (Event incomingEvent)
{
    if (incomingEvent != _event)
    {
        if (_successor != 0)
        {
            cout << "HandleEvent () calling into successor\n";
            _successor->HandleEvent(incomingEvent);
        }
    }
    else
    {
        cout << "Base class help now follows";
    }
}

class Connection : public EventHandler {
public:
	Connection(Event t) : EventHandler(0, t) { }

	virtual void HandleEvent(Event incomingEvent);
};

void Connection::HandleEvent(Event incomingEvent)
{
	if (HasEventSupport(incomingEvent)) {
	// Offer event support
	cout << "Here's some Connection Event support\n";
    cout << "You may need additonal bandwidth\n\n";
	}
	else
	{
		cout << "No support from Connection - Sorry!\n";
		cout << "Calling the base class event handler\n";
		EventHandler::HandleEvent(incomingEvent);
	}
}

class Path : public Connection {
public:
	Path::Path(Connection* w, Event t) : Connection(0) {
		cout << "In the Path constructor\n";
		SetHandler(w, t);
	}

	void HandleEvent(Event incomingEvent)
	{
		if (HasEventSupport(incomingEvent)) {
			// Offer event support
			cout << "Here's some Path Event support\n";
			cout << "We need a new path\n\n";
		}
		else
		{
			cout << "No support from Path - Sorry!\n";
			cout << "Calling the base class event handler\n";
			EventHandler::HandleEvent(incomingEvent);
		}
	}
};

class Lsp : public Connection {
public:
	Lsp::Lsp(char* lspID, Connection* w, Event t) : Connection(0) {
		cout << "Constructing Lsp: " << lspID << "\n";
		lspName = lspID;
		SetHandler(w, t);
	}

	void HandleEvent(Event incomingEvent) {
		if (HasEventSupport(incomingEvent)) {
			// Offer event support
			cout << "\nAt last, here's some LSP Event support:\n";
			cout << "We need a new path for " << lspName << "\n\n";
		}
		else
			EventHandler::HandleEvent(incomingEvent);
	}

private:
	char* lspName;
};

void main()
    {
	Connection* connection = new Connection(CONGESTION);
	Lsp* lsp = new Lsp("LSP123", connection, LINK_1_BROKEN);
	Path* path = new Path(lsp, LINK_2_BROKEN);

	//A network event has occurred, e.g., CONGESTION
    //Let's see who can handle this event
    
    cout << "Let's simulate a network error: CONGESTION\n";
    path->HandleEvent(CONGESTION);

	//Another network event has occurred, e.g., LINK_1_BROKEN
    //Let's see who can handle this event
    
    cout << "Let's simulate another network error: LINK_1_BROKEN\n";
    path->HandleEvent(LINK_1_BROKEN);

    cout << "Let's simulate one more network error: LINK_2_BROKEN\n";
    path->HandleEvent(LINK_2_BROKEN);

    delete path;
    delete lsp;
    delete connection;
}